import lights.PointLight
import materials.MaterialColor
import materials.PathTracedMaterial
import materials.ReflectiveMaterial
import materials.WhateverMaterial
import materials.times
import math.Point
import math.Vector
import things.Plane
import things.Scene
import things.Sphere
import java.io.File


fun main() {
    val bmp = Bitmap(1000, 1000)
    val origin = Point(500f, 500f, -500f)

    val material1 = PathTracedMaterial(
        MaterialColor(1f, 0f, 0f),
        specularProbability = 0.1f,
        specularColour = MaterialColor.White,
        smoothness = 1.0f,
    )

    val materialFloor = PathTracedMaterial(
        MaterialColor(1f, 0f, 1f),
        specularProbability = 0.1f,
        specularColour = MaterialColor.White,
        smoothness = 1.0f,
    )

    val lightMaterial = PathTracedMaterial(
        color = MaterialColor.Black,
        emissionColor = MaterialColor.White,
        emissionStrength = 0.2f,
        specularColour = MaterialColor.White
    )

    val material2 = ReflectiveMaterial(reflectivity = 0.2f, MaterialColor(0f, 1.0f, 0f))
    val material3 = ReflectiveMaterial(reflectivity = 0.2f, MaterialColor(1.0f, 0.0f, 0.1f))

    val sphere = Sphere(Point(500f, 500f, 500f), radius = 500f, material = material1)
    val sphere2 = Sphere(Point(0f, 500f, 50f), radius = 50f, material = material1)
    val sphere3 = Sphere(Point(1000f, 1000f, -500f), radius = 100f, material = lightMaterial)

    val material4 = WhateverMaterial(MaterialColor(1f, 0.0f, 0.0f))
    val plane = Plane(
        Point(0f, -1000f, 0f),
        normal = Vector(0f, 1f, 0f),
        material = materialFloor
    )

    val material5 = WhateverMaterial(MaterialColor(0.0f, 1f, 0.0f))
    val leftPlane = Plane(
        Point(-1000f, 0f, 0f),
        Vector(1f, 0f, 0f),
        material = materialFloor
    )

    val light = PointLight(Point(1000f, 1000f, -500f), color = 0.2f * MaterialColor(0.9f, 0.9f, 0.9f))

    val scene = Scene(listOf(
        sphere,
        sphere2,
        plane,
        leftPlane,
        sphere3
    ), light = light, ambientLight = MaterialColor.White * 0.1f)


    for (y in 0 until bmp.height) {
        for (x in 0 until bmp.width) {
            var color = MaterialColor.Black

            repeat(samplesPerPixel) {
                val point = Point(x.toFloat(), y.toFloat(), 0f) + Vector.randomInCircleXY() * blurriness
                val ray = origin.rayTo(point)

                color += scene.colorForRay(ray)
            }

            bmp.setPixel(
                x, y,
                (color * (1.0f / samplesPerPixel)).toColor()
            )
        }
    }
    File("test.tga").writeBitmap(bmp)
}

const val samplesPerPixel = 64
const val blurriness = 1f